package com.simaek.notify.dingtalk;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.simaek.notify.DefaultNotifyType;
import com.simaek.notify.Notifier;
import com.simaek.notify.NotifyType;
import com.simaek.notify.Provider;
import okhttp3.*;

import java.io.IOException;
import java.time.Duration;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

public class DingTalkNotifier implements Notifier<DingTalkMessageTemplate> {

    private final AtomicReference<String> accessToken = new AtomicReference<>();

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://oapi.dingtalk.com/gettoken";

    private static final String notifyApi = "https://oapi.dingtalk.com/topapi/message/corpconversation/asyncsend_v2";

    private final DingTalkProperties properties;

    private final String notifierId;

    private final OkHttpClient httpClient;

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    public DingTalkNotifier(String id, DingTalkProperties properties) {
        this.properties = properties;
        this.notifierId = id;
        this.httpClient = new OkHttpClient.Builder().build();
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.dingTalk;
    }

    @Override
    public Provider getProvider() {
        return DingTalkProvider.dingTalkMessage;
    }

    @Override
    public void send(DingTalkMessageTemplate template, Map<String, Object> context) {
        HttpUrl url = Objects.requireNonNull(HttpUrl.parse(notifyApi)).newBuilder()
                .addQueryParameter("access_token", getToken())
                .build();
        String formInserter = template.createFormInserter(context);
        RequestBody requestBody = RequestBody.create(formInserter, MediaType.get("application/json"));
        Request request = new Request.Builder()
                .url(url)
                .post(requestBody)
                .build();
        Call call = httpClient.newCall(request);
        try {
            Response response = call.execute();
            checkResult(Objects.requireNonNull(response.body()).string());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void checkResult(String msg) {
        JSONObject jsonObject = JSON.parseObject(msg);
        int errorCode = jsonObject.getIntValue("errcode");
        if (errorCode != 0) {
            throw new RuntimeException("发送钉钉通知失败:" + jsonObject.get("errmsg"));
        }
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken.get() == null) {
            return requestToken();
        }
        return accessToken.get();
    }

    private String requestToken() {
        HttpUrl.Builder urlBuilder = Objects.requireNonNull(HttpUrl.parse(tokenApi)).newBuilder();
        urlBuilder.addQueryParameter("appkey", properties.getAppKey());
        urlBuilder.addQueryParameter("appsecret", properties.getAppSecret());
        Request request = new Request.Builder()
                .url(urlBuilder.build())
                .get()
                .build();
        Call call = httpClient.newCall(request);
        try {
            Response response = call.execute();
            JSONObject responseJson = JSON.parseObject(Objects.requireNonNull(response.body()).string());
            if (responseJson.containsKey("access_token")) {
                String access_token = responseJson.get("access_token").toString();
                refreshTokenTime = System.currentTimeMillis();
                accessToken.set(access_token);
                return access_token;
            } else {
                throw new RuntimeException("获取Token失败:" + responseJson.get("errmsg"));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void close() {
        accessToken.set(null);
        refreshTokenTime = 0;
    }
}
