package com.simaek.notify.email.embedded;

import com.alibaba.fastjson.JSONObject;
import com.simaek.notify.*;
import com.simaek.notify.email.EmailProvider;
import com.simaek.notify.email.EmailTemplate;
import com.simaek.notify.email.EmailTemplateParsed;
import com.simaek.notify.util.ExpressionUtils;
import com.simaek.notify.util.StringUtils;
import okhttp3.Call;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.InputStreamSource;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * 使用javax.mail进行邮件发送
 *
 * @author xueye
 **/
public class DefaultEmailNotifier implements Notifier<EmailTemplate> {

    private JavaMailSender javaMailSender;

    public JavaMailSender getJavaMailSender() {
        return javaMailSender;
    }

    public void setJavaMailSender(JavaMailSender javaMailSender) {
        this.javaMailSender = javaMailSender;
    }

    private String sender;

    public String getSender() {
        return sender;
    }

    public void setSender(String sender) {
        this.sender = sender;
    }

    private final String notifierId;

    private final OkHttpClient httpClient;

    @Override
    public String getNotifierId() {
        return notifierId;
    }

//    public static Scheduler scheduler = Schedulers.newElastic("email-notifier");

    public DefaultEmailNotifier(NotifierProperties properties) {
        notifierId = properties.getId();
        DefaultEmailProperties emailProperties = new JSONObject(properties.getConfiguration()).toJavaObject(DefaultEmailProperties.class);
        JavaMailSenderImpl mailSender = new JavaMailSenderImpl();
        mailSender.setHost(emailProperties.getHost());
        mailSender.setPort(emailProperties.getPort());
        mailSender.setUsername(emailProperties.getUsername());
        mailSender.setPassword(emailProperties.getPassword());
        mailSender.setJavaMailProperties(emailProperties.createJavaMailProperties());
        this.sender = emailProperties.getSender();
        this.javaMailSender = mailSender;
        this.httpClient = new OkHttpClient.Builder().build();
    }

    @Override
    public void send(EmailTemplate template, Map<String, Object> context) {
        EmailTemplateParsed emailTemplateParsed = convert(template, context);
        try {
            this.doSend(emailTemplateParsed, template.getSendTo());
        } catch (MessagingException | UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void close() {
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.email;
    }

    @Override
    public Provider getProvider() {
        return EmailProvider.embedded;
    }

    protected void doSend(EmailTemplateParsed template, List<String> sendTo) throws MessagingException, UnsupportedEncodingException {
        MimeMessage mimeMessage = this.javaMailSender.createMimeMessage();
        MimeMessageHelper helper = new MimeMessageHelper(mimeMessage, true, "utf-8");

        helper.setFrom(this.sender);
        helper.setTo(sendTo.toArray(new String[0]));
        helper.setSubject(template.getSubject());
        helper.setText(new String(template.getText().getBytes(), StandardCharsets.UTF_8), true);

        Set<Map.Entry<String, String>> entries = template.getAttachments().entrySet();
        for (Map.Entry<String, String> entry : entries) {
            helper.addAttachment(MimeUtility.encodeText(entry.getKey()), convertResource(entry.getValue()));
        }
        Set<Map.Entry<String, String>> imageEntries = template.getImages().entrySet();
        for (Map.Entry<String, String> image : imageEntries) {
            helper.addInline(image.getKey(), convertResource(image.getValue()), "application/octet-stream");
        }
        this.javaMailSender.send(mimeMessage);
    }


    protected InputStreamSource convertResource(String resource) {
        if (resource.startsWith("http://") || resource.startsWith("https://")) {
            Request request = new Request.Builder()
                    .url(resource)
                    .get()
                    .header("Accept", "application/octet-stream")
                    .build();
            Call call = httpClient.newCall(request);
            try {
                Response response = call.execute();
                InputStream inputStream = Objects.requireNonNull(response.body()).byteStream();
                new InputStreamResource(inputStream);
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else {
            try {
                return new InputStreamResource(new FileInputStream(resource));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }
        return null;
    }


    protected EmailTemplateParsed convert(EmailTemplate template, Map<String, Object> context) {
        String subject = template.getSubject();
        String text = template.getText();
        if (StringUtils.isEmpty(subject) || StringUtils.isEmpty(text)) {
            throw new RuntimeException("模板内容错误，text 或者 subject 不能为空.");
        }
        String sendText = render(text, context);
        List<EmailTemplate.Attachment> tempAttachments = template.getAttachments();
        Map<String, String> attachments = new HashMap<>();
        if (tempAttachments != null) {
            for (EmailTemplate.Attachment tempAttachment : tempAttachments) {
                attachments.put(tempAttachment.getName(), render(tempAttachment.getLocation(), context));
            }
        }
        return EmailTemplateParsed.builder()
                .attachments(attachments)
                .images(extractSendTextImage(sendText))
                .text(sendText)
                .subject(render(subject, context))
                .build();
    }


    private Map<String, String> extractSendTextImage(String sendText) {
        Map<String, String> images = new HashMap<>();
        Document doc = Jsoup.parse(sendText);
        for (Element src : doc.getElementsByTag("img")) {
            String s = src.attr("src");
            if (s.startsWith("http")) {
                continue;
            }
            String tempKey = UUID.randomUUID().toString();
            src.attr("src", "cid:".concat(tempKey));
            images.put(tempKey, s);
        }
        return images;
    }

    private String render(String str, Map<String, Object> context) {
        return ExpressionUtils.analytical(str, context);
    }

}
