package com.simaek.notify.sms.telecom;

import com.alibaba.fastjson.JSONObject;
import com.simaek.notify.*;
import com.simaek.notify.sms.SmsProvider;
import com.simaek.notify.util.ExpressionUtils;
import okhttp3.*;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * 云信通短信通知平台通知器
 *
 * @author xueye
 */
public class TelecomSmsNotifier implements Notifier<TelecomSmsTemplate> {

    private static final String notifyApi = "http://www.js139.com.cn:8022/hysms/SendMsg";

    private final TelecomSmsNotifierConfiguration configuration;

    private final String notifierId;

    private final OkHttpClient httpClient;

    public TelecomSmsNotifier(NotifierProperties properties) {
        this.notifierId = properties.getId();
        this.configuration = new JSONObject(properties.getConfiguration()).toJavaObject(TelecomSmsNotifierConfiguration.class);
        this.httpClient = new OkHttpClient.Builder().build();
    }

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.sms;
    }

    @Override
    public Provider getProvider() {
        return SmsProvider.js139;
    }

    @Override
    public void send(TelecomSmsTemplate template, Map<String, Object> context) {
        try {
            HttpUrl url = Objects.requireNonNull(HttpUrl.parse(notifyApi)).newBuilder()
                    .addQueryParameter("userId", configuration.getUserId())
                    .addQueryParameter("password", configuration.getPassword())
                    .addQueryParameter("content", render(template.getContent(), context))
                    .addQueryParameter("mobile", template.getMobile())
                    .build();
            Request request = new Request.Builder()
                    .url(url)
                    .post(RequestBody.create(new byte[0]))
                    .build();
            Call call = httpClient.newCall(request);
            Response response = call.execute();
            Map<String, String> resultMap = handleResponse(Objects.requireNonNull(response.body()).string());
            String resultCodeKey = "rspCode";
            if (!resultMap.containsKey(resultCodeKey) || (!"0".equals(resultMap.get(resultCodeKey)) && !"DELIVRD".equalsIgnoreCase(resultMap.get(resultCodeKey)))) {
                throw new RuntimeException("短信发送失败：" + resultMap.get("rspDesc"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void close() {
    }

    /**
     * 使用变量替和SpEL表达式渲染模板
     *
     * @param expression 带有变量或者表达式的模板
     * @param context    用于替换的上下文
     * @return 渲染后的模板
     */
    private String render(String expression, Map<String, Object> context) {
        return ExpressionUtils.analytical(expression, context);
    }

    /**
     * 处理HTTP请求响应信息
     * <p>将URL参数转换成键值对</p>
     *
     * @param response 响应字符串
     * @return 处理过的响应信息
     */
    private Map<String, String> handleResponse(String response) {
        Map<String, String> map = new HashMap<>();
        String[] param = response.split("&");
        for (String key : param) {
            String[] pair = key.split("=");
            if (pair.length == 2) {
                map.put(pair[0], pair[1]);
            }
        }
        return map;
    }
}
