package com.simaek.notify.wechat.corp;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.simaek.notify.DefaultNotifyType;
import com.simaek.notify.Notifier;
import com.simaek.notify.NotifyType;
import com.simaek.notify.Provider;
import com.simaek.notify.wechat.WechatProvider;
import okhttp3.*;

import java.io.IOException;
import java.time.Duration;
import java.util.Map;
import java.util.Objects;

public class WechatCorpNotifier implements Notifier<WechatCorpTemplate> {

    private String accessToken;

    private long refreshTokenTime;

    private final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    private static final String tokenApi = "https://qyapi.weixin.qq.com/cgi-bin/gettoken";

    private static final String notifyApi = "https://qyapi.weixin.qq.com/cgi-bin/message/send";

    private final WechatCorpProperties properties;

    private final String notifierId;

    private final OkHttpClient httpClient;

    public WechatCorpNotifier(String id, WechatCorpProperties properties) {
        this.properties = properties;
        this.notifierId = id;
        this.httpClient = new OkHttpClient.Builder().build();
    }

    @Override
    public String getNotifierId() {
        return notifierId;
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.wechat;
    }

    @Override
    public Provider getProvider() {
        return WechatProvider.corpMessage;
    }

    @Override
    public void send(WechatCorpTemplate template, Map<String, Object> context) {
        HttpUrl url = Objects.requireNonNull(HttpUrl.parse(notifyApi)).newBuilder()
                .addQueryParameter("access_token", getToken())
                .build();
        Request request = new Request.Builder()
                .url(url)
                .post(RequestBody.create(template.createJsonRequest(context), MediaType.get("application/json")))
                .header("Content-Type", "application/json;charset=utf8")
                .build();
        try {
            Call call = httpClient.newCall(request);
            Response response = call.execute();
            String result = Objects.requireNonNull(response.body()).string();
            JSONObject jsonObject = JSON.parseObject(result);
            if (!"0".equals(jsonObject.get("errcode").toString())) {
                throw new RuntimeException("发送微信企业通知失败：" + jsonObject.get("errmsg"));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken == null) {
            return requestToken();
        }
        return accessToken;
    }

    private String requestToken() {
        HttpUrl url = Objects.requireNonNull(HttpUrl.parse(tokenApi)).newBuilder()
                .addQueryParameter("corpId", properties.getCorpId())
                .addQueryParameter("corpSecret", properties.getCorpSecret())
                .build();
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();
        try {
            Call call = httpClient.newCall(request);
            Response response = call.execute();
            JSONObject responseJson = JSON.parseObject(Objects.requireNonNull(response.body()).string());
            if (responseJson.containsKey("access_token")) {
                String access_token = responseJson.get("access_token").toString();
                accessToken = access_token;
                refreshTokenTime = System.currentTimeMillis();
                return access_token;
            } else {
                throw new RuntimeException("获取令牌失败：" + responseJson.getString("errmsg"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public void close() {
        this.accessToken = null;
        this.refreshTokenTime = 0;
    }
}
