package com.simaek.notify.wechat.subscription;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.simaek.notify.*;
import com.simaek.notify.wechat.WechatProvider;
import okhttp3.*;

import java.io.IOException;
import java.time.Duration;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.atomic.AtomicReference;

/**
 * 微信公众号消息通知器
 *
 * @author xueye
 */
public class WechatSubscriptionNotifier implements Notifier<WechatSubscriptionTemplate> {

    /**
     * 通知器配置信息
     */
    private final NotifierProperties properties;

    /**
     * 全局唯一接口调用凭证
     */
    private final AtomicReference<String> accessToken = new AtomicReference<>();

    /**
     * 获取凭证的Api
     */
    private static final String tokenApi = "https://api.weixin.qq.com/cgi-bin/token";

    /**
     * 发送消息通知的Api
     */
    private static final String notifyApi = "https://api.weixin.qq.com/cgi-bin/message/template/send";

    /**
     * 凭证有效时间
     */
    private static final long tokenTimeOut = Duration.ofSeconds(7000).toMillis();

    /**
     * 凭据刷新时间
     * <p>一般接口对凭证的获取会有次数限制，记录上一次获取凭证的时间，每次发送通知的时候，配合凭证有效时间来判断是否重新获取</p>
     */
    private long refreshTokenTime;

    private final OkHttpClient httpClient;

    public WechatSubscriptionNotifier(NotifierProperties properties) {
        this.properties = properties;
        this.httpClient = new OkHttpClient.Builder().build();
    }

    @Override
    public String getNotifierId() {
        return properties.getId();
    }

    @Override
    public NotifyType getType() {
        return DefaultNotifyType.wechat;
    }

    @Override
    public Provider getProvider() {
        return WechatProvider.subscription;
    }

    @Override
    public void send(WechatSubscriptionTemplate template, Map<String, Object> context) {
        HttpUrl url = Objects.requireNonNull(HttpUrl.parse(notifyApi)).newBuilder()
                .addQueryParameter("access_token", getToken())
                .build();
        Request request = new Request.Builder()
                .url(url)
                .header("Content-Type", "application/json; charset=UTF-8")
                .post(RequestBody.create(template.createJsonRequest(), MediaType.get("application/json")))
                .build();
        try {
            Call call = httpClient.newCall(request);
            Response response = call.execute();
            JSONObject responseJson = JSON.parseObject(Objects.requireNonNull(response.body()).string());
            System.out.println(responseJson);
            if (!responseJson.containsKey("errcode") || responseJson.getIntValue("errcode") != 0) {
                throw new RuntimeException(responseJson.get("errmsg").toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void close() {
        accessToken.set(null);
        refreshTokenTime = 0;
    }

    private String getToken() {
        if (System.currentTimeMillis() - refreshTokenTime > tokenTimeOut || accessToken.get() == null) {
            refreshToken();
        }
        return accessToken.get();
    }

    /**
     * 刷新接口访问凭证
     */
    private void refreshToken() {
        WechatSubscriptionProperties config = new JSONObject(properties.getConfiguration()).toJavaObject(WechatSubscriptionProperties.class);
        HttpUrl url = HttpUrl.parse(tokenApi).newBuilder()
                .addQueryParameter("grant_type", config.getGrantType())
                .addQueryParameter("appid", config.getAppId())
                .addQueryParameter("secret", config.getSecret())
                .build();
        Request request = new Request.Builder()
                .url(url)
                .get()
                .build();
        try {
            Call call = httpClient.newCall(request);
            Response response = call.execute();
            JSONObject responseJson = JSON.parseObject(Objects.requireNonNull(response.body()).string());
            if (responseJson.containsKey("access_token")) {
                accessToken.set(responseJson.get("access_token").toString());
            } else if (responseJson.containsKey("errcode")) {
                throw new RuntimeException(responseJson.get("errmsg").toString());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
